/*
 * Copyright (c) International Business Machines  Corp., 2005
 * Copyright (c) Wipro Technologies Ltd, 2005.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */
/**********************************************************
 *
 *    TEST IDENTIFIER   : getpagesize01
 *
 *    EXECUTED BY       : root / superuser
 *
 *    TEST TITLE        : Basic tests for getpagesize(2)
 *
 *    TEST CASE TOTAL   : 1
 *
 *    AUTHOR            : Prashant P Yendigeri 
 *                        <prashant.yendigeri@wipro.com> 
 *			  Robbie Williamson
 *			  <robbiew@us.ibm.com>
 *
 *    DESCRIPTION
 *      This is a Phase I test for the getpagesize(2) system call.
 *      It is intended to provide a limited exposure of the system call.
 *
 **********************************************************/

#include <stdio.h>
#include <unistd.h>
#include <errno.h>

#include "test.h"
#include "usctest.h"

void setup();
void cleanup();

char *TCID="getpagesize01";     /* Test program identifier.    */
int TST_TOTAL=1;                /* Total number of test cases. */
extern int Tst_count;           /* Test Case counter for tst_* routines */

int exp_enos[]={0};             /* must be a 0 terminated list */

int  main(int ac, char **av)
{
    	int lc;             /* loop counter */
	char *msg;          /* message returned from parse_opts */

	int size, ret_sysconf;
	/***************************************************************
	 * parse standard options
	 ***************************************************************/
	if ( (msg=parse_opts(ac, av, (option_t *) NULL, NULL)) != (char *) NULL )
        tst_brkm(TBROK, cleanup, "OPTION PARSING ERROR - %s", msg);

    	/***************************************************************
     	* perform global setup for test
     	***************************************************************/
    	setup();

    	/* set the expected errnos... */
    	TEST_EXP_ENOS(exp_enos);

    	/***************************************************************
     	* check looping state if -c option given
     	***************************************************************/
    	for (lc=0; TEST_LOOPING(lc); lc++) {

        	/* reset Tst_count in case we are looping. */
        	Tst_count=0;


        	/*
         	* TEST CASE:
         	*  Get page size
         	*/
        	;

        	/* Call getpagesize(2) */
        	TEST(getpagesize());

        /* check return code */
        if ( TEST_RETURN == -1 ) {
            TEST_ERROR_LOG(TEST_ERRNO);
            tst_resm(TFAIL, "getpagesize -  Get page size failed, errno=%d : %s",
                     TEST_ERRNO, strerror(TEST_ERRNO));
            continue;   /* next loop for MTKERNEL */
        }

        /***************************************************************
         * only perform functional verification if flag set (-f not given)
         ***************************************************************/
        if ( STD_FUNCTIONAL_TEST ) {
	  size = getpagesize();
	  tst_resm(TINFO,"Page Size is %d",size);
	  ret_sysconf = sysconf(_SC_PAGESIZE);
#ifdef DEBUG
	  tst_resm(TINFO,"Checking whether getpagesize returned same as sysconf");
#endif
	  if(size == ret_sysconf)
	    tst_resm(TPASS,"getpagesize - Page size returned %d",ret_sysconf);
	  else
	    tst_resm(TFAIL,"getpagesize - Page size returned %d",ret_sysconf);
        }
    }   /* End for TEST_LOOPING */

    /***************************************************************
     * cleanup and exit
     ***************************************************************/
    cleanup();

    return 0;
}       /* End main */

/***************************************************************
 * setup() - performs all ONE TIME setup for this test.
 ***************************************************************/
void
setup()
{
    /* capture signals */
    tst_sig(NOFORK, DEF_HANDLER, cleanup);

    /* Pause if that option was specified */
    TEST_PAUSE;
}       /* End setup() */



/***************************************************************
 * cleanup() - performs all ONE TIME cleanup for this test at
 *              completion or premature exit.
 ***************************************************************/
void
cleanup()
{
    /*
     * print timing stats if that option was specified.
     * print errno log if that option was specified.
     */
    TEST_CLEANUP;

    /* exit with return code appropriate for results */
    tst_exit();
}       /* End cleanup() */

